@@ -13,6 +13,7 @@
 # under the License.
 
 
+import json
 import os
 import warnings
 
@@ -44,15 +45,16 @@ CONFIG_SEARCH_PATH = [
     SITE_CONFIG_HOME, UNIX_SITE_CONFIG_HOME
 ]
 YAML_SUFFIXES = ('.yaml', '.yml')
+JSON_SUFFIXES = ('.json',)
 CONFIG_FILES = [
     os.path.join(d, 'clouds' + s)
     for d in CONFIG_SEARCH_PATH
-    for s in YAML_SUFFIXES
+    for s in YAML_SUFFIXES + JSON_SUFFIXES
 ]
 VENDOR_FILES = [
     os.path.join(d, 'clouds-public' + s)
     for d in CONFIG_SEARCH_PATH
-    for s in YAML_SUFFIXES
+    for s in YAML_SUFFIXES + JSON_SUFFIXES
 ]
 
 BOOL_KEYS = ('insecure', 'cache')
@@ -212,16 +214,19 @@ class OpenStackConfig(object):
                 'expiration', self._cache_expiration)
 
     def _load_config_file(self):
-        return self._load_yaml_file(self._config_files)
+        return self._load_yaml_json_file(self._config_files)
 
     def _load_vendor_file(self):
-        return self._load_yaml_file(self._vendor_files)
+        return self._load_yaml_json_file(self._vendor_files)
 
-    def _load_yaml_file(self, filelist):
+    def _load_yaml_json_file(self, filelist):
         for path in filelist:
             if os.path.exists(path):
                 with open(path, 'r') as f:
-                    return path, yaml.safe_load(f)
+                    if path.endswith('json'):
+                        return path, json.load(f)
+                    else:
+                        return path, yaml.safe_load(f)
         return (None, None)
 
     def _normalize_keys(self, config):
@@ -230,6 +235,12 @@ class OpenStackConfig(object):
             key = key.replace('-', '_')
             if isinstance(value, dict):
                 new_config[key] = self._normalize_keys(value)
+            elif isinstance(value, bool):
+                new_config[key] = value
+            elif isinstance(value, int):
+                new_config[key] = str(value)
+            elif isinstance(value, float):
+                new_config[key] = str(value)
             else:
                 new_config[key] = value
         return new_config
@@ -270,7 +281,17 @@ class OpenStackConfig(object):
                     " parameter in {0} instead.".format(self.config_filename))
             return regions
         else:
-            return ['']
+            # crappit. we don't have a region defined.
+            new_cloud = dict()
+            our_cloud = self.cloud_config['clouds'].get(cloud, dict())
+            self._expand_vendor_profile(cloud, new_cloud, our_cloud)
+            if 'regions' in new_cloud and new_cloud['regions']:
+                return new_cloud['regions']
+            elif 'region_name' in new_cloud and new_cloud['region_name']:
+                return [new_cloud['region_name']]
+            else:
+                # Wow. We really tried
+                return ['']
 
     def _get_region(self, cloud=None):
         return self._get_regions(cloud)[0]
@@ -291,7 +312,18 @@ class OpenStackConfig(object):
 
         # Get the defaults
         cloud.update(self.defaults)
+        self._expand_vendor_profile(name, cloud, our_cloud)
 
+        if 'auth' not in cloud:
+            cloud['auth'] = dict()
+
+        _auth_update(cloud, our_cloud)
+        if 'cloud' in cloud:
+            del cloud['cloud']
+
+        return self._fix_backwards_madness(cloud)
+
+    def _expand_vendor_profile(self, name, cloud, our_cloud):
         # Expand a profile if it exists. 'cloud' is an old confusing name
         # for this.
         profile_name = our_cloud.get('profile', our_cloud.get('cloud', None))
@@ -314,15 +346,6 @@ class OpenStackConfig(object):
                                   " the cloud '{1}'".format(profile_name,
                                                             name))
 
-        if 'auth' not in cloud:
-            cloud['auth'] = dict()
-
-        _auth_update(cloud, our_cloud)
-        if 'cloud' in cloud:
-            del cloud['cloud']
-
-        return self._fix_backwards_madness(cloud)
-
     def _fix_backwards_madness(self, cloud):
         cloud = self._fix_backwards_project(cloud)
         cloud = self._fix_backwards_auth_plugin(cloud)
@@ -394,11 +417,14 @@ class OpenStackConfig(object):
         return cloud
 
     def _fix_backwards_interface(self, cloud):
+        new_cloud = {}
         for key in cloud.keys():
             if key.endswith('endpoint_type'):
                 target_key = key.replace('endpoint_type', 'interface')
-                cloud[target_key] = cloud.pop(key)
-        return cloud
+            else:
+                target_key = key
+            new_cloud[target_key] = cloud[key]
+        return new_cloud
 
     def get_all_clouds(self):
 
@@ -615,6 +641,13 @@ class OpenStackConfig(object):
                 if type(config[key]) is not bool:
                     config[key] = get_boolean(config[key])
 
+        # TODO(mordred): Special casing auth_url here. We should
+        #                come back to this betterer later so that it's
+        #                more generalized
+        if 'auth' in config and 'auth_url' in config['auth']:
+            config['auth']['auth_url'] = config['auth']['auth_url'].format(
+                **config)
+
         # NOTE(dtroyer): OSC needs a hook into the auth args before the
         #                plugin is loaded in order to maintain backward-
         #                compatible behaviour
